==> liquidsoap_2.0.6.orig.tar.bz2: liquidsoap-2.0.6/src/operators/midi_routing.ml <==
(*****************************************************************************

  Liquidsoap, a programmable audio stream generator.
  Copyright 2003-2022 Savonet team

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details, fully stated in the COPYING
  file at the root of the liquidsoap distribution.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA

 *****************************************************************************)

open Mm
open Source

class virtual base ~kind ~name (source : source) =
  object
    inherit operator ~name kind [source]
    method stype = source#stype
    method remaining = source#remaining
    method seek = source#seek
    method self_sync = source#self_sync
    method is_ready = source#is_ready
    method abort_track = source#abort_track
  end

class merge ~kind (source : source) out =
  object
    inherit base ~kind source ~name:"midi.merge_all"

    method private get_frame buf =
      source#get buf;
      let m = MFrame.midi buf in
      for c = 0 to Array.length m - 1 do
        MIDI.merge m.(out) m.(c);
        if c <> out then MIDI.clear_all m.(c)
      done
  end

class remove ~kind (source : source) t =
  object
    inherit base ~kind source ~name:"midi.remove"

    method private get_frame buf =
      source#get buf;
      let m = MFrame.midi buf in
      List.iter (fun c -> if c < Array.length m then MIDI.clear_all m.(c)) t
  end

let () =
  let kind = { Frame.audio = `Any; video = `Any; midi = Frame.midi_native } in
  let k = Lang.kind_type_of_kind_format kind in
  Lang.add_operator "midi.merge_all"
    [
      ("track_out", Lang.int_t, Some (Lang.int 0), Some "Destination track.");
      ("", Lang.source_t k, None, None);
    ]
    ~return_t:k ~category:`MIDI ~descr:"Merge all MIDI tracks in one."
    (fun p ->
      let f v = List.assoc v p in
      let out = Lang.to_int (f "track_out") in
      let src = Lang.to_source (f "") in
      let kind = Source.Kind.of_kind kind in
      new merge ~kind src out)

let () =
  let kind = { Frame.audio = `Any; video = `Any; midi = Frame.midi_native } in
  let k = Lang.kind_type_of_kind_format kind in
  Lang.add_operator "midi.remove"
    [
      ("", Lang.list_t Lang.int_t, None, Some "Tracks to remove.");
      ("", Lang.source_t k, None, None);
    ]
    ~return_t:k ~category:`MIDI ~descr:"Remove MIDI tracks."
    (fun p ->
      (* let f v = List.assoc v p in *)
      let t = List.map Lang.to_int (Lang.to_list (Lang.assoc "" 1 p)) in
      let src = Lang.to_source (Lang.assoc "" 2 p) in
      let kind = Source.Kind.of_kind kind in
      new remove ~kind src t)

==> liquidsoap_2.1.3.orig.tar.bz2: liquidsoap-2.1.3/src/operators/midi_routing.ml <==
(*****************************************************************************

  Liquidsoap, a programmable audio stream generator.
  Copyright 2003-2022 Savonet team

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details, fully stated in the COPYING
  file at the root of the liquidsoap distribution.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA

 *****************************************************************************)

open Mm
open Source

class virtual base ~kind ~name (source : source) =
  object
    inherit operator ~name kind [source]
    method stype = source#stype
    method remaining = source#remaining
    method seek = source#seek
    method self_sync = source#self_sync
    method is_ready = source#is_ready
    method abort_track = source#abort_track
  end

class merge ~kind (source : source) out =
  object
    inherit base ~kind source ~name:"midi.merge_all"

    method private get_frame buf =
      source#get buf;
      let m = MFrame.midi buf in
      for c = 0 to Array.length m - 1 do
        MIDI.merge m.(out) m.(c);
        if c <> out then MIDI.clear_all m.(c)
      done
  end

class remove ~kind (source : source) t =
  object
    inherit base ~kind source ~name:"midi.remove"

    method private get_frame buf =
      source#get buf;
      let m = MFrame.midi buf in
      List.iter (fun c -> if c < Array.length m then MIDI.clear_all m.(c)) t
  end

let () =
  let kind = { Frame.audio = `Any; video = `Any; midi = Frame.midi_native } in
  let k = Lang.kind_type_of_kind_format kind in
  Lang.add_operator "midi.merge_all"
    [
      ("track_out", Lang.int_t, Some (Lang.int 0), Some "Destination track.");
      ("", Lang.source_t k, None, None);
    ]
    ~return_t:k ~category:`MIDI ~descr:"Merge all MIDI tracks in one."
    (fun p ->
      let f v = List.assoc v p in
      let out = Lang.to_int (f "track_out") in
      let src = Lang.to_source (f "") in
      let kind = Kind.of_kind kind in
      new merge ~kind src out)

let () =
  let kind = { Frame.audio = `Any; video = `Any; midi = Frame.midi_native } in
  let k = Lang.kind_type_of_kind_format kind in
  Lang.add_operator "midi.remove"
    [
      ("", Lang.list_t Lang.int_t, None, Some "Tracks to remove.");
      ("", Lang.source_t k, None, None);
    ]
    ~return_t:k ~category:`MIDI ~descr:"Remove MIDI tracks."
    (fun p ->
      (* let f v = List.assoc v p in *)
      let t = List.map Lang.to_int (Lang.to_list (Lang.assoc "" 1 p)) in
      let src = Lang.to_source (Lang.assoc "" 2 p) in
      let kind = Kind.of_kind kind in
      new remove ~kind src t)
